import struct
#import smbus
from smbus2 import SMBusWrapper as sbu


LSM303_ADDRESS_ACCEL = 0x1D 
LSM303_ADDRESS_MAG   = 0x1E
                                         # Default    Type
LSM303_REGISTER_ACCEL_CTRL_REG1_A = 0x20 # 00000111   rw
LSM303_REGISTER_ACCEL_CTRL_REG4_A = 0x23 # 00000000   rw
LSM303_REGISTER_ACCEL_OUT_X_L_A   = 0x28
LSM303_REGISTER_ACCEL_STATUS_REG_A= 0x27
#LSM303_REGISTER_MAG_CRB_REG_M     = 0x01
#LSM303_REGISTER_MAG_MR_REG_M      = 0x02
LSM303_REGISTER_MAG_CTRL_REG1_M    = 0x20 # 00010000   rw
LSM303_REGISTER_MAG_OUT_X_H_M      = 0x28

ACCEL_REG1 = 0b00011111
ACCEL_REG4 = 0b00111000
MAG_REG1   = 0b00010000 #ODR: out data rate 10Hz


class LSM303C(object):
    n_bus=1
    def __init__(self, hires=True, accel_add=LSM303_ADDRESS_ACCEL, mag_add=LSM303_ADDRESS_MAG):
        with sbu(n_bus) as bus:
            # Enable the accelerometer
            self.bus.write_byte_data(accel_add,LSM303_REGISTER_ACCEL_CTRL_REG1_A, ACCEL_REG1)
            # Select hi-res (12-bit) or low-res (10-bit) output mode.
            # Low-res mode uses less power and sustains a higher update rate,
            # output is padded to compatible 12-bit units.
            if hires:
                self.bus.write_byte_data(accel_add,LSM303_REGISTER_ACCEL_CTRL_REG4_A, ACCEL_REG4)
            else:
                self.bus.write_byte_data(accel_add,LSM303_REGISTER_ACCEL_CTRL_REG4_A, 0x00)
            # Enable the magnetometer
            self.bus.write_byte_data(mag_add,LSM303_REGISTER_MAG_CTRL_REG1_M, MAG_REG1)
            #self.bus.write_byte_data(mag_add,LSM303_REGISTER_MAG_CTRL_REG1_M, MAG_REG1)

    def lee(self, accel_add=LSM303_ADDRESS_ACCEL, mag_add=LSM303_ADDRESS_MAG):
        #Read the accelerometer and magnetometer value.  A tuple of tuples will be returned with:((accel X, accel Y, accel Z), (mag X, mag Y, mag Z))"
        with sbu(n_bus) as bus:
            # Read the accelerometer as signed 16-bit little endian values.
            accel_raw = self.bus.read_i2c_block_data(accel_add,LSM303_REGISTER_ACCEL_OUT_X_L_A , 6)
            accel = struct.unpack('<hhh', accel_raw)
            # Convert to 12-bit values by shifting unused bits.
            accel = (accel[0] >> 4, accel[1] >> 4, accel[2] >> 4)
            # Read the magnetometer.
            mag_raw = self.read_i2c_block_data(mag_add,LSM303_REGISTER_MAG_OUT_X_H_M, 6)
            mag = struct.unpack('>hhh', mag_raw)
            return (accel, mag)
        
    def lee_acc(self, accel_add=LSM303_ADDRESS_ACCEL):
        with sbu(n_bus) as bus:
            # Read the accelerometer as signed 16-bit little endian values.
            accel_raw = self.bus.read_i2c_block_data(accel_add,LSM303_REGISTER_ACCEL_OUT_X_L_A , 6)
            accel = struct.unpack('<hhh', accel_raw)
            # Convert to 12-bit values by shifting unused bits.
            accel = (accel[0] >> 4, accel[1] >> 4, accel[2] >> 4)
            return accel
        
    def lee_kmp(self, mag_add=LSM303_ADDRESS_MAG):
        with sbu(n_bus) as bus:
            # Read the magnetometer.
            mag_raw = self.bus.read_i2c_block_data(mag_add,LSM303_REGISTER_MAG_OUT_X_H_M, 6)
            mag = struct.unpack('>hhh', mag_raw)
            return mag
